@@ -1,6 +1,8 @@
 import collections
 import datetime
 import logging
+import matplotlib
+from matplotlib import pyplot
 import numpy
 import pathlib
 import pytest
@@ -97,7 +99,7 @@ class TestBasicCalibrationModel:
         with pytest.raises(NotImplementedError):
             _ = cmodel.predict_independent(x)
         with pytest.raises(NotImplementedError):
-            _ = cmodel.infer_independent(y, lower=0, upper=10, steps=10, hdi_prob=None)
+            _ = cmodel.infer_independent(y, lower=0, upper=10, steps=10, ci_prob=None)
         with pytest.raises(NotImplementedError):
             _ = cmodel.loglikelihood(y=y, x=x, theta=[1,2,3])
         pass
@@ -321,7 +323,7 @@ class TestModelFunctions:
 class TestSymbolicModelFunctions:
     def _check_numpy_theano_equivalence(self, function, theta):
         # make sure that test value computation is turned off (pymc3 likes to turn it on)
-        with theano.configparser.change_flags(compute_test_value='off'):
+        with theano.config.change_flags(compute_test_value='off'):
             # create computation graph
             x = tt.vector('x', dtype=theano.config.floatX)
             y = function(x, theta)
@@ -439,14 +441,14 @@ class TestUtils:
 
     @pytest.mark.skipif(HAS_PYMC3, reason='run only if PyMC3 is not installed')
     def test_has_modules(self):
-        assert not (calibr8.utils.HAS_THEANO)
-        assert not (calibr8.utils.HAS_PYMC3)
+        assert not calibr8.utils.HAS_TENSORS
+        assert not calibr8.utils.HAS_PYMC3
         return
 
     @pytest.mark.skipif(not HAS_PYMC3, reason='requires PyMC3')
     def test_has_modules(self):
-        assert (calibr8.utils.HAS_THEANO)
-        assert (calibr8.utils.HAS_PYMC3)
+        assert calibr8.utils.HAS_TENSORS
+        assert calibr8.utils.HAS_PYMC3
         return
 
     def test_assert_version_match(self):
@@ -471,55 +473,19 @@ class TestUtils:
             calibr8.utils.assert_version_match("1.1.1.1", "1.1.1.2")
         return
 
-    def test_guess_asymmetric_logistic_theta(self):
-        with pytest.raises(ValueError):
-            calibr8.guess_asymmetric_logistic_theta([1,2,3], [1,2])
-        with pytest.raises(ValueError):
-            calibr8.guess_asymmetric_logistic_theta([1,2], [[1,2],[2,3]])
-        L_L, L_U, I_x, S, c = calibr8.guess_asymmetric_logistic_theta(
-            X=[0, 1, 2, 3, 4, 5],
-            Y=[0, 1, 2, 3, 4, 5],
-        )
-        assert L_L == 0
-        assert L_U == 10
-        assert I_x == (0+5)/2
-        numpy.testing.assert_almost_equal(S, 1)
-        assert c == -1
-        return
+    @pytest.mark.parametrize("residual_type", ["relative", "absolute"])
+    def test_plot_model(self, residual_type):
+        em = _TestPolynomialModel(independent_key='S', dependent_key='A365', mu_degree=1, scale_degree=1)
+        em.theta_fitted = [0, 2, 0.1, 1]
+        em.cal_independent = numpy.linspace(0.1, 10, 7)
+        mu, scale, df = em.predict_dependent(em.cal_independent)
+        em.cal_dependent = scipy.stats.t.rvs(loc=mu, scale=scale, df=df)
 
-    def test_guess_asymmetric_logistic_bounds(self):
-        with pytest.raises(ValueError):
-            calibr8.guess_asymmetric_logistic_theta([1,2,3], [1,2])
-        with pytest.raises(ValueError):
-            calibr8.guess_asymmetric_logistic_theta([1,2], [[1,2],[2,3]])
-        
-        for half_open in (True, False):
-            L_L, L_U, I_x, S, c = calibr8.guess_asymmetric_logistic_bounds(
-                X=[0, 1, 2, 3, 4, 5],
-                Y=[0, 1, 2, 3, 4, 5],
-                half_open=half_open
-            )
-            if half_open:
-                numpy.testing.assert_allclose(L_L, (-numpy.inf, 2.5))
-                numpy.testing.assert_allclose(L_U, (2.5, numpy.inf))
-            else:
-                numpy.testing.assert_allclose(L_L, (0-100*5, 2.5))
-                numpy.testing.assert_allclose(L_U, (2.5, 5+100*5))
-            numpy.testing.assert_allclose(I_x, (-15, 20))
-            numpy.testing.assert_allclose(S, (0, 10))
-            numpy.testing.assert_allclose(c, (-5, 5))
-        return
-
-    def test_guess_asymmetric_logistic_theta_in_bounds(self):
-        X = numpy.linspace(0, 50, 42)
-        Y = numpy.random.normal(X, scale=0.4)
-        theta = calibr8.guess_asymmetric_logistic_theta(X, Y)
-        for half_open in (True, False):
-            bounds = calibr8.guess_asymmetric_logistic_bounds(X, Y, half_open=half_open)
-            for t, (lb, ub) in zip(theta, bounds):
-                assert t > lb
-                assert t < ub
-        return
+        fig, axs = calibr8.utils.plot_model(em, residual_type=residual_type)
+        assert isinstance(fig, matplotlib.figure.Figure)
+        assert numpy.shape(axs) == (3,)
+        pyplot.close()
+        pass
 
 
 class TestContribBase:
@@ -658,32 +624,38 @@ class TestBasePolynomialModelT:
     def test_infer_independent(self):
         em = _TestPolynomialModel(independent_key='S', dependent_key='A365', mu_degree=1, scale_degree=1)
         em.theta_fitted = [0, 2, 0.1, 1]
-        x, pdf, median, hdi_prob, lower_x, upper_x = em.infer_independent(y=1, lower=0, upper=20, steps=876)
+        pst = em.infer_independent(y=1, lower=0, upper=20, steps=876)
 
-
-        assert len(x) == len(pdf)
-        assert x[0] == 0
-        assert x[-1] == 20
-        assert (numpy.isclose(scipy.integrate.cumtrapz(pdf,x)[-1], 1, atol=0.0001))
-        assert (lower_x==0)
-        assert (upper_x==20)
-        assert (hdi_prob==1)
+        assert len(pst.eti_x) == len(pst.eti_pdf)
+        assert len(pst.hdi_x) == len(pst.hdi_pdf)
+        assert tuple(pst.eti_x[[0, -1]]) == (0, 20)
+        assert tuple(pst.hdi_x[[0, -1]]) == (0, 20)
+        assert (numpy.isclose(scipy.integrate.cumtrapz(pst.eti_pdf, pst.eti_x)[-1], 1, atol=0.0001))
+        assert (numpy.isclose(scipy.integrate.cumtrapz(pst.hdi_pdf, pst.hdi_x)[-1], 1, atol=0.0001))
+        assert pst.eti_lower == pst.hdi_lower == 0
+        assert pst.eti_upper == pst.hdi_upper == 20
+        assert pst.eti_prob == 1
+        assert pst.hdi_prob == 1
 
         # check trimming to [2.5,97.5] interval
-        posterior = em.infer_independent(y=1, lower=0, upper=20, steps=1775, hdi_prob=0.95)
+        pst = em.infer_independent(y=[1, 2], lower=0, upper=20, steps=1775, ci_prob=0.95)
 
-        assert (len(posterior)==6)
-        assert len(posterior.x_dense) == len(posterior.pdf)
-        assert (posterior.hdi_prob==0.95)
-        assert (numpy.isclose(scipy.integrate.cumtrapz(posterior.pdf,posterior.x_dense)[-1], 0.95, atol=0.0001))
-        assert posterior.hdi_lower == posterior.x_dense[0]
-        assert posterior.hdi_upper == posterior.x_dense[-1]
+        assert len(pst.eti_x) == len(pst.eti_pdf)
+        assert len(pst.hdi_x) == len(pst.hdi_pdf)
+        assert numpy.isclose(pst.eti_prob, 0.95, atol=0.0001)
+        assert numpy.isclose(pst.hdi_prob, 0.95, atol=0.0001)
+        assert numpy.isclose(scipy.integrate.cumtrapz(pst.eti_pdf, pst.eti_x)[-1], 0.95, atol=0.0001)
+        assert numpy.isclose(scipy.integrate.cumtrapz(pst.hdi_pdf, pst.hdi_x)[-1], 0.95, atol=0.0001)
+        assert pst.eti_lower == pst.eti_x[0]
+        assert pst.eti_upper == pst.eti_x[-1]
+        assert pst.hdi_lower == pst.hdi_x[0]
+        assert pst.hdi_upper == pst.hdi_x[-1]
 
         # check that error are raised by wrong input
         with pytest.raises(ValueError):
-            _ = em.infer_independent(y=1, lower=0, upper=20, steps=1000, hdi_prob=(-1))
+            _ = em.infer_independent(y=1, lower=0, upper=20, steps=1000, ci_prob=(-1))
         with pytest.raises(ValueError):
-            _ = em.infer_independent(y=1, lower=0, upper=20, steps=1000, hdi_prob=(97.5))
+            _ = em.infer_independent(y=1, lower=0, upper=20, steps=1000, ci_prob=(97.5))
         pass
     
     @pytest.mark.skipif(not HAS_PYMC3, reason='requires PyMC3')
@@ -707,33 +679,63 @@ class TestBasePolynomialModelT:
             'x_hat': x_test
         })
         expected = cmodel.loglikelihood(x=x_test, y=y_obs)
+        assert numpy.ndim(actual) == numpy.ndim(expected) == 0
         numpy.testing.assert_almost_equal(actual, expected, 6)
         pass
 
-    def test_loglikelihood(self):
-        x = numpy.array([1,2,3])
-        y = numpy.array([1,2,3])
+    @pytest.mark.parametrize("x", [
+        numpy.array([1,2,3]),
+        4,
+    ])
+    @pytest.mark.parametrize("y", [
+        numpy.array([2,4,8]),
+        5,
+    ])
+    def test_loglikelihood(self, x, y):
         cmodel = _TestPolynomialModel(independent_key='S', dependent_key='OD', mu_degree=1, scale_degree=1)
         cmodel.theta_fitted = [0, 1, 0.1, 1.6]
-        with pytest.raises(TypeError):
-            _ = cmodel.loglikelihood(y, x=x)
-        true = cmodel.loglikelihood(y=y, x=x)
+
+        actual = cmodel.loglikelihood(y=y, x=x)
+        assert numpy.ndim(actual) == 0
         mu, scale, df = cmodel.predict_dependent(x, theta=cmodel.theta_fitted)
         expected = numpy.sum(stats.t.logpdf(x=y, loc=mu, scale=scale, df=df))
-        assert expected == true
-        x = 'hello'
-        with pytest.raises(Exception):
-            _= cmodel.loglikelihood(y=y, x=x)
+        numpy.testing.assert_equal(actual, expected)
         return
-    
-    def test_loglikelihood_without_fit(self):
-        x = numpy.array([1,2,3])
-        y = numpy.array([1,2,3])
-        cmodel = _TestPolynomialModel(independent_key='Glu', dependent_key='OD', mu_degree=1, scale_degree=1)
-        with pytest.raises(Exception):
-            _= cmodel.loglikelihood(y=y, x=x)
+
+    def test_loglikelihood_exceptions(self):
+        cmodel = _TestPolynomialModel(independent_key='S', dependent_key='OD', mu_degree=1, scale_degree=1)
+        with pytest.raises(Exception, match="No parameter vector"):
+            cmodel.loglikelihood(y=[2,3], x=[4,5])
+
+        cmodel.theta_fitted = [0, 1, 0.1, 1.6]
+
+        with pytest.raises(TypeError):
+            cmodel.loglikelihood(4, x=[2,3])
+        with pytest.raises(ValueError, match="Input x must be"):
+            cmodel.loglikelihood(y=[2,3], x="hello")
+        with pytest.raises(ValueError, match="operands could not be broadcast"):
+            cmodel.loglikelihood(y=[1,2,3], x=[1,2])
         return
 
+    def test_likelihood(self):
+        # use a linear model with intercept 1 and slope 0.5
+        cmodel = _TestPolynomialModel(independent_key="I", dependent_key="D", mu_degree=1)
+        cmodel.theta_fitted = [1, 0.5, 0.5]
+
+        assert numpy.isscalar(cmodel.likelihood(y=2, x=3))
+        assert numpy.isscalar(cmodel.likelihood(y=[2, 3], x=[3, 4]))
+
+        with pytest.raises(ValueError, match="operands could not be broadcast"):
+            cmodel.likelihood(y=[1,2,3], x=[1,2])
+
+        x_dense = numpy.linspace(0, 4, 501)
+        actual = cmodel.likelihood(x=x_dense, y=2, scan_x=True)
+        assert numpy.ndim(actual) == 1
+        # the maximum likelihood should be at x=2
+        assert x_dense[numpy.argmax(actual)] == 2
+        pass
+
+
 
 class TestBaseAsymmetricLogisticModelT:
     def test_predict_dependent(self):
@@ -757,54 +759,6 @@ class TestBaseAsymmetricLogisticModelT:
         x_predicted = cmodel.predict_independent(y=mu)
         assert (numpy.allclose(x_predicted, x_original))
         return
-    
-    @pytest.mark.skipif(not HAS_PYMC3, reason='requires PyMC3')
-    def test_symbolic_loglikelihood(self):
-        cmodel = _TestLogisticModel(independent_key='S', dependent_key='A', scale_degree=1)
-        cmodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.23]
-       
-        # create test data
-        x_true = numpy.array([1,2,3,4,5])
-        y_obs = cmodel.predict_dependent(x_true)[0]
-
-        # create a pymc3 model using the calibration model
-        with pymc3.Model() as pmodel:
-            x_hat = pymc3.Uniform('x_hat', lower=0, upper=10, shape=x_true.shape, transform=None)
-            L = cmodel.loglikelihood(x=x_hat, y=y_obs, replicate_id='A01', dependent_key='A')
-            assert isinstance(L, tt.TensorVariable)
-        
-        # compare the two loglikelihood computation methods
-        x_test = numpy.random.normal(x_true, scale=0.1)
-        actual = L.logp({
-            'x_hat': x_test
-        })
-        expected = cmodel.loglikelihood(x=x_test, y=y_obs)
-        numpy.testing.assert_almost_equal(actual, expected, 6)
-        return
-    
-    def test_loglikelihood(self):
-        x = numpy.array([1,2,3])
-        y = numpy.array([1,2,3])
-        cmodel = _TestLogisticModel(independent_key='S', dependent_key='OD', scale_degree=1)
-        cmodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.7]
-        with pytest.raises(TypeError):
-            _ = cmodel.loglikelihood(y, x=x)
-        true = cmodel.loglikelihood(y=y, x=x)
-        mu, scale, df = cmodel.predict_dependent(x, theta=cmodel.theta_fitted)
-        expected = numpy.sum(stats.t.logpdf(x=y, loc=mu, scale=scale, df=df))
-        assert expected == true
-        x = 'hello'
-        with pytest.raises(Exception):
-            _= cmodel.loglikelihood(y=y, x=x)
-        return
-    
-    def test_loglikelihood_without_fit(self):
-        x = numpy.array([1,2,3])
-        y = numpy.array([1,2,3])
-        cmodel = _TestLogisticModel(independent_key='Glu', dependent_key='OD', scale_degree=1)
-        with pytest.raises(Exception):
-            _= cmodel.loglikelihood(y=y, x=x)
-        return
 
 
 class TestOptimization:
